function animateTestEngine(mdl,trainData,testData,engineNum)
% Inspect a residual similarity model's performance on new test data by
% animating it over time in a series of subplots.

% create figure and subplots
f=figure('Visible','on','Position',get(0,'ScreenSize'));
ax1 = subplot(3,2,[1,3],'Parent',f);
ax2 = subplot(3,2,[2,4],'Parent',f);
ax3 = subplot(3,2,[5,6],'Parent',f);

% preallocate variables
ttf = -length(testData)+1:0;
rulAnswer = length(testData)-1:-1:0;
estRUL = nan(length(testData),1);
ciRUL = nan(length(testData),2);
pdfRUL = cell(length(testData),1);
histRUL = cell(length(testData),1);
[estRUL(1), ciRUL(1,:), pdfRUL{1}, histRUL{1}] = predictRUL(mdl, testData(1));

% set up first subplot
hold(ax1,'on')
rulA = plot(ax1,ttf(1),rulAnswer(1),'g','LineWidth',1); % answer
rulE = plot(ax1,ttf(1),estRUL(1),'Color',[0.8500,0.3250,0.0980],'LineWidth',1); % estimate
ciL = plot(ax1,ttf(1),ciRUL(1,1),'--k','LineWidth',1); % lower CI bound
ciU = plot(ax1,ttf(1),ciRUL(1,2),'--k','LineWidth',1); % upper CI bound
ciPatch = patch(ax1,[ttf(1),ttf(1)], [ciRUL(1,1),ciRUL(1,2)], 'y', ...
    'FaceAlpha', 0.2, 'EdgeColor', 'none'); % CI patch
hold(ax1,'off')
title(ax1,"Test Engine "+engineNum)
xlabel(ax1,"Cycles Prior to Failure")
ylabel(ax1,"Remaining Useful Life Prediction (Cycles)")
legend([rulA,rulE,ciPatch],["Test Answer";"Model Prediction";"90% CI"])

% set up second subplot
set(f,'CurrentAxes',ax2)
hold(ax2,'on')
cellNum = num2cell((1:length(trainData))');
ptdata = cell2mat(cellfun(@(x,y) [[(1:length(x))';NaN],[x;NaN],ones(length(x)+1,1)*y],trainData,cellNum,'UniformOutput',false));
ptColor = [0.3010,0.7450,0.9330];
pt = plot(ax2,ptdata(:,1),ptdata(:,2),'Color',ptColor,'LineWidth',0.2);
idxKNN = knnsearch(mdl,testData(1));
idxpknn = ismember(ptdata(:,3),idxKNN);
pknn = plot(ax2,ptdata(idxpknn,1),ptdata(idxpknn,2),'Color',[0.4660,0.6740,0.1880],'LineWidth',1);
pmdata = cellfun(@(x) [length(x),x(end)],trainData(idxKNN),'UniformOutput',false);
pmdata = cell2mat(pmdata);
pm = plot(ax2,pmdata(:,1),pmdata(:,2),'*k');
ptest = plot(ax2,testData(1),'Color','r','LineWidth',2);
hold(ax2,'off')
xlabel(ax2,'Engine Runtime (cycles)')
ylabel(ax2,'Engine Health Indicator')
title(ax2,'K=50 Nearest Training Engine Trajetories')
legend('Training Engines','K Nearest Engines','Engine Failures','Test Engine','AutoUpdate','off');

% set up third subplot
set(f,'CurrentAxes',ax3)
hold(ax3,'on')
xlabel('Remaining Useful Life (cycles)')
yyaxis left
h = histogram(ax3,histRUL{1}.RUL,20);
ylabel('Count of Training Engine RUL')
yyaxis right
ylabel('PDF of Training Engine RUL')
ksp = plot(ax3,pdfRUL{1}.RUL,pdfRUL{1}.ProbabilityDensity,'Color',[0.9290    0.6940    0.1250]);
idx = find(pdfRUL{1}.RUL > rulAnswer(1), 1, 'first');
if isempty(idx)
    y = pdfRUL{1}.ProbabilityDensity(end);
else
    y = pdfRUL{1}.ProbabilityDensity(idx);
end
trueAns = plot(ax3,[rulAnswer(1), rulAnswer(1)], [0, y], '-g','LineWidth',2);
idx = find(pdfRUL{1}.RUL > estRUL(1), 1, 'first');
if isempty(idx)
    y = pdfRUL{1}.ProbabilityDensity(end);
else
    y = pdfRUL{1}.ProbabilityDensity(idx);
end
estAns = plot(ax3,[estRUL(1), estRUL(1)], [0, y], '-','Color',[0.8500,0.3250,0.0980],'LineWidth',2);
idx = pdfRUL{1}.RUL >= ciRUL(1,1) & pdfRUL{1}.RUL<=ciRUL(1,2);
ciArea = area(ax3,pdfRUL{1}.RUL(idx), pdfRUL{1}.ProbabilityDensity(idx), ...
    'FaceAlpha', 0.2, 'FaceColor', 'y', 'EdgeColor', 'none');
legend('Training RUL','RUL PDF','Test Answer','Model Prediction','90%CI')
title('Histogram and KSDensity Plot of Training Engine RUL')
hold(ax3,'off')

% main loop
for ii = 2:length(testData)
    % make prediction
    [estRUL(ii), ciRUL(ii,:), pdfRUL{ii}, histRUL{ii}] = predictRUL(mdl, testData(1:ii));

    % update first subplot
    set(rulA,'XData',ttf(1:ii),'YData',rulAnswer(1:ii))
    set(rulE,'XData',ttf(1:ii),'YData',estRUL(1:ii))
    set(ciL,'XData',ttf(1:ii),'YData',ciRUL(1:ii,1))
    set(ciU,'XData',ttf(1:ii),'YData',ciRUL(1:ii,2))
    set(ciPatch,'XData',[ttf(1:ii),ttf(ii:-1:1)],'YData',[ciRUL(1:ii,1)',ciRUL(ii:-1:1,2)'])

    % update second subplot
    idxKNN = knnsearch(mdl,testData(1:ii));
    idxpknn = ismember(ptdata(:,3),idxKNN);
    set(pknn,'XData',ptdata(idxpknn,1),'YData',ptdata(idxpknn,2))
    pmdata = cellfun(@(x) [length(x),x(end)],trainData(idxKNN),'UniformOutput',false);
    pmdata = cell2mat(pmdata);
    set(pm,'XData',pmdata(:,1),'YData',pmdata(:,2))
    set(ptest,'YData',testData(1:ii))

    % update third subplot
    set(h,'Data',histRUL{ii}.RUL,'NumBins',20)
    set(ksp,'XData',pdfRUL{ii}.RUL,'YData',pdfRUL{ii}.ProbabilityDensity)
    idx = find(pdfRUL{ii}.RUL > rulAnswer(ii), 1, 'first');
    if isempty(idx)
        y = pdfRUL{ii}.ProbabilityDensity(end);
    else
        y = pdfRUL{ii}.ProbabilityDensity(idx);
    end
    set(trueAns,'XData',[rulAnswer(ii),rulAnswer(ii)],'YData',[0,y])
    idx = find(pdfRUL{ii}.RUL > estRUL(ii), 1, 'first');
    if isempty(idx)
        y = pdfRUL{ii}.ProbabilityDensity(end);
    else
        y = pdfRUL{ii}.ProbabilityDensity(idx);
    end
    set(estAns,'XData',[estRUL(ii),estRUL(ii)],'YData',[0,y])
    idx = pdfRUL{ii}.RUL >= ciRUL(ii,1) & pdfRUL{ii}.RUL<=ciRUL(ii,2);
    set(ciArea,'XData',pdfRUL{ii}.RUL(idx),'YData',pdfRUL{ii}.ProbabilityDensity(idx))

    % force graphics update
    drawnow
end