classdef rescaleLayer < nnet.layer.Layer
    
    properties
        a
        b
    end
    
    methods
        function this = rescaleLayer( xMax, xMin, yMax, yMin )
            % linearRescaleLayer   rescale data using linear transformation
            %
            %               Y = a*X + b
            %
            % such that:    yMax = a*xMax + b
            %               yMin = a*xMin + b
            
            this.a = (yMax(:) - yMin(:))./(xMax(:) - xMin(:));
            this.b = (xMax(:).*yMin(:) - xMin(:).*yMax(:))./(xMax(:) - xMin(:));
        end
        
        function Z = predict(this, X)
            % Simple linear transformation
            N = size(X, 2);
            S = size(X, 3);
            A = repmat( this.a, 1, N, S );
            B = repmat( this.b, 1, N, S );
            Z = A.*X + B;
        end
        
        function dX = backward(this, ~, ~, dZ, ~)
            N = size(dZ, 2);
            S = size(dZ, 3);
            A = repmat( this.a, 1, N, S );
            dX = A.*dZ;
        end
    end
end