classdef weightedCrossEntropyLayer < nnet.layer.ClassificationLayer
    
    properties
        ClassWeights
    end
    
    methods
        function this = weightedCrossEntropyLayer( responseCategorical )
            % Determine weights based on training response distribution
            if iscell( responseCategorical )
                % Assume seq2seq cell array of categorical response
                % sequences
                responseCategorical = [ responseCategorical{:} ];
            end
            h = histogram( responseCategorical );
            xlabel('Classes')
            ylabel('Class Count')
            weights = numel( responseCategorical )./h.Values;
            weights = weights./sum(weights);
            this.ClassWeights = weights(:);
            
        end
        
        function loss = forwardLoss(this, Y, T)
            numElems = size(Y, 2)*size(Y, 3);
            W = repmat( this.ClassWeights, 1, size(T, 2), size(T, 3) );
            loss = -sum( sum( sum( W.*T.*log(iBoundAwayFromZero(Y)) ) ) )./numElems;
        end
        
        function dX = backwardLoss( this, Y, T)
            numObs = size(Y, 2);
            W = repmat( this.ClassWeights, 1, size(T, 2), size(T, 3) );
            dX = -W.*T./iBoundAwayFromZero(Y);
            dX = dX./numObs;
        end
    end
end

function xBounded = iBoundAwayFromZero(x)
xBounded = x;
precision = iGetPrecision(x);
xBounded(xBounded < eps(precision)) = eps(precision);
end

function precision = iGetPrecision(x)
if(isa(x,'gpuArray'))
    precision = classUnderlying(x);
else
    precision = class(x);
end
end